@@ -1,6 +1,6 @@
 /* $RCSfile: infer.c,v $
--- $Revision: 1.4 $
--- last change: $Author: rt $ $Date: 2004/09/08 16:06:32 $
+-- $Revision: 1.8 $
+-- last change: $Author: ihi $ $Date: 2007/10/15 15:39:49 $
 --
 -- SYNOPSIS
 --      Infer how to make a target.
@@ -37,7 +37,7 @@
 
 #define A_TRANSFER  (A_EPILOG | A_PRECIOUS | A_SILENT | A_SHELL | A_SETDIR |\
 		     A_SEQ | A_LIBRARY | A_IGNORE | A_PROLOG | A_SWAP |\
-		     A_NOSTATE )
+		     A_PHONY | A_NOSTATE )
 
 
 /* Define local static functions */
@@ -52,6 +52,11 @@ static ICELLPTR   add_iset    ANSI((ICELLPTR,ICELLPTR,CELLPTR,DFALINKPTR,
 static ICELLPTR   derive_prerequisites ANSI((ICELLPTR, ICELLPTR *));
 static char *	  dump_inf_chain ANSI((ICELLPTR, int, int));
 
+#ifdef DBUG
+static void	  _dump_dfa_stack ANSI((DFALINKPTR, DFASETPTR));
+static void	  _dump_iset ANSI(( char *, ICELLPTR ));
+#endif
+
 
 PUBLIC void
 Infer_recipe( cp, setdirroot )/*
@@ -67,6 +72,8 @@ CELLPTR setdirroot;
 
    if( cp->ce_attr & A_NOINFER ) {DB_VOID_RETURN;}
 
+   DB_PRINT("inf", ("Inferring rule for [%s]", cp->CE_NAME));
+
    match = NIL(ICELL);
    nomatch = add_iset( NIL(ICELL), NIL(ICELL), NIL(CELL), NIL(DFALINK),
 			setdirroot, Prep+count_dots(cp->CE_NAME), 0,
@@ -78,11 +85,12 @@ CELLPTR setdirroot;
 
    DB_EXECUTE( "inf", _dump_iset("nomatch",nomatch); );
 
+   /* If nomatch is non-empty there was no match with an existing
+    * prerrequisite, try to derive one. */
    while( nomatch != NIL(ICELL) ) {
       ICELLPTR new_nomatch = NIL(ICELL);
       ICELLPTR ic, pmatch, mmatch;
       CELLPTR  prereq;
-      int      first;
 
       for( ic=nomatch; ic != NIL(ICELL); ic=ic->ic_next ) {
 	 int ipush = FALSE;
@@ -107,7 +115,10 @@ CELLPTR setdirroot;
        */
       if( match == NIL(ICELL) ) {
 	 nomatch = new_nomatch;
+
+	 /* Skip the rest and try one level deeper. */
 	 if( Transitive ) continue;
+
 	 goto all_done;
       }
 
@@ -128,6 +139,10 @@ CELLPTR setdirroot;
        * prerequisites that already exist. */
       pmatch = mmatch = NIL(ICELL);
       for(; match; match = ic ) {
+	 /* This loop checks all possible matches. */
+	 DB_PRINT("inf", ("Target [%s] : prerequisite [%s]",
+			  match->ic_meta->CE_NAME, match->ic_name));
+
 	 ic = match->ic_next;
 	 match->ic_next = NIL(ICELL);
 
@@ -137,38 +152,31 @@ CELLPTR setdirroot;
 	    mmatch = union_iset(mmatch, match);
       }
 
+      /* Prefer %-targets with existing prerequisites. */
       if( pmatch )
 	 match = pmatch;
       else
 	 match = mmatch;
 
-      /* Make sure it is unique */
+      /* Make sure it is unique. It would be easy to check
+       * match->ic_meta->ce_prq for existence and prefer no prerequisites
+       * over prerequisites that are present, but we are currently not
+       * doing it. */
       if( match->ic_next != NIL(ICELL) ) {
-	 int dump = (match->ic_next->ic_next != NIL(ICELL));
+	 int count = 1;
 
-	 /* Check for definite ambiguity */
-	 if( !dump ) {
-	    if( (match->ic_meta->ce_prq && match->ic_next->ic_meta->ce_prq) ||
-	        (!match->ic_meta->ce_prq && !match->ic_next->ic_meta->ce_prq)  )
-	       dump = TRUE;
-	    else if(!match->ic_meta->ce_prq && match->ic_next->ic_meta->ce_prq )
-	       match = match->ic_next;
-     }
-     else {
-	    int count = 1;
-
-	    Continue = TRUE;
-	    Error( "Ambiguous inference chains for target '%s'", cp->CE_NAME );
-	    for( ic=match; ic; ic=ic->ic_next )
-	       (void) dump_inf_chain(ic, TRUE, count++);
-	    Fatal( "resolve ambiguity before proceeding.");
-	    /*NOTREACHED*/
-	 }
+	 Warning( "Ambiguous inference chains for target '%s'", cp->CE_NAME );
+	 for( ic=match; ic; ic=ic->ic_next )
+	    (void) dump_inf_chain(ic, TRUE, count++);
+	 Warning( "First matching rule is chosen.");
       }
 
-      /* MATCH now points at the derived recipe.  We must now take cp, and
-       * construct the correct graph so that the make may proceed. */
+      /* MATCH now points at the derived prerequisite chain(s).  We must now
+       * take cp, and construct the correct graph so that the make may
+       * proceed.  */
 
+      /* The folowing shows only the first element, i.e. the last matching
+       * recipe that was found.  */ 
       if( Verbose & V_INFER ) {
 	 char *tmp = dump_inf_chain(match, TRUE, FALSE);
 	 printf("%s:  Inferring prerequistes and recipes using:\n%s:  ... %s\n",
@@ -177,8 +185,8 @@ CELLPTR setdirroot;
 
       pmatch = NIL(ICELL);
       prereq = NIL(CELL);
-      first  = TRUE;
 
+      /* This loop treats the inferred targets last to first. */
       while( match ) {
          CELLPTR infcell=NIL(CELL);
 
@@ -192,14 +200,13 @@ CELLPTR setdirroot;
 	    infcell->ce_flag |= F_TARGET;
 
 	    if( infcell != cp ) {
-	       infcell->ce_flag |= F_INFER;
-	       if( !first ) infcell->ce_flag |= F_REMOVE;
+	       infcell->ce_flag |= F_INFER|F_REMOVE;
+	       DB_PRINT("remove", ("Mark for deletion [%s]",
+				   infcell->CE_NAME));
 	    }
 
 	    if( !match->ic_flag )
 	       infcell->ce_attr |= A_NOINFER;
-
-	    first = FALSE;
 	 }
 
 	 /* Add global prerequisites from previous rule if there are any and
@@ -208,9 +215,21 @@ CELLPTR setdirroot;
 	    CELLPTR imeta = pmatch->ic_meta;
 	    LINKPTR lp;
 
+	    DB_PRINT("inf", ("%%-target [%s] - infered target [%s]\n",
+			     imeta->CE_NAME, infcell->CE_NAME));
+
 	    infcell->ce_per   = pmatch->ic_dfa->dl_per;
 	    infcell->ce_attr |= (imeta->ce_attr & A_TRANSFER);
 
+	    /* The .PHONY mechanism relies on having phony targets not
+	     * being STATed and having a zero time stamp. While inferring
+	     * the this target it might have been created and stated
+	     * therefore these values need to be reset. */
+	    if( infcell->ce_attr & A_PHONY ){
+	       infcell->ce_time =  0L;
+	       infcell->ce_flag &= ~F_STAT;
+	    }
+
 	    if( !(infcell->ce_flag & F_RULES) ) {
 	       infcell->ce_flag |= (imeta->ce_flag&(F_SINGLE|F_GROUP))|F_RULES;
 	       infcell->ce_recipe = imeta->ce_recipe;
@@ -236,20 +255,30 @@ CELLPTR setdirroot;
 
 	    pmatch->ic_dfa->dl_per = NIL(char);
 
-	    /* If infcell already had a directory set then modify it based on
-	     * whether it was the original cell or some intermediary. */
+	    /* If infcell already had a .SETDIR directory set then modify it
+	     * based on whether it was the original cell or some intermediary. */
 	    if( imeta->ce_dir ) {
 	       if( infcell->ce_dir && infcell == cp ) {
 		  /* cp->ce_dir was set and we have pushed the directory prior
-		   * to calling this routine.  We should therefore pop it and
-		   * push the new concatenated directory required by the
-		   * inference. */
-		  infcell->ce_dir=DmStrDup(Build_path(infcell->ce_dir,
-						     imeta->ce_dir));
+		   * to calling this routine.
+		   * We build a new path by appending imeta->ce_dir to the
+		   * current directory of the original cell.
+		   * We should therefore pop it and push the new concatenated
+		   * directory required by the inference.
+		   * This leaks memory as cp->ce_dir is not freed before
+		   * setting the new the new infcell->ce_dir value but as
+		   * the pointer could be a `A_POOL` member we accept this. */
+		  infcell->ce_dir = DmStrDup(Build_path(infcell->ce_dir,
+							imeta->ce_dir));
 	       }
-	       else
-		  infcell->ce_dir = imeta->ce_dir;
-        }
+	       else {
+		  /* Inherit a copy of the .SETDIR value. Use a copy because
+		   * the original could have been freed in the meantime
+		   * in Make() by the FREE() before _pool_lookup(). This can
+		   * also leak if infcell->ce_dir was set before. */
+		  infcell->ce_dir = DmStrDup(imeta->ce_dir);
+	       }
+	    }
 
 	    for( lp=imeta->ce_indprq; lp != NIL(LINK); lp=lp->cl_next ) {
 	       char    *name = lp->cl_prq->CE_NAME;
@@ -271,8 +300,10 @@ CELLPTR setdirroot;
 	 if( prereq )
 	    (Add_prerequisite(infcell,prereq,FALSE,FALSE))->cl_flag |=F_TARGET;
 
-	 pmatch = match;
-	 prereq = infcell;
+	 pmatch = match;   /* Previous member in inference chain ... */
+	 prereq = infcell; /* is a prerequisite to the next match.   */
+	 /* ip->ic_parent is the next target in the inference chain to be
+	  * build. If it is empty we are done. */
 	 match  = match->ic_parent;
       }
 
@@ -308,6 +339,8 @@ ICELLPTR *nnmp;
 
    DB_ENTER("derive_prerequisites");
 
+   DB_PRINT("inf", ("for [%s]\n", ic->ic_name));
+
    /* If none of the inference nodes match then forget about the inference.
     * The user did not tell us how to make such a target.  We also stop the
     * Inference if the new set of DFA's is a proper subset of a previous
@@ -368,6 +401,7 @@ ICELLPTR *nnmp;
 	 int      noinf;
 	 int	  exists;
 	 
+	 /* Name of the prerequisite, can be empty. */
 	 if( meta->ce_prq )
 	    name = meta->ce_prq->cl_prq->CE_NAME;
 
@@ -409,7 +443,25 @@ ICELLPTR *nnmp;
 	    thp = Get_name( iprqh.ht_name, Defs, FALSE );
 	    if(thp != NIL(HASH)) {
 	       iprq = *thp->CP_OWNR;
-	       ircp = iprq.ce_recipe;
+	       /* Check if a recipe for this target exists. Targets with F_MULTI
+		* set need each cell checked for existing recipes.
+		*/
+	       if( iprq.ce_flag & F_MULTI ) {
+		  /* Walk through all cells of this target. */
+		  LINKPTR mtcp = iprq.ce_prq;
+		  ircp = NIL(STRING);
+		  for( ; mtcp != NIL(LINK); mtcp = mtcp->cl_next ) {
+		     /* If a recipe is found stop searching and set ircp to that result.
+		      * ircp is not used but only checked if it is set.
+		      */
+		     if( mtcp->cl_prq->ce_recipe != NIL(STRING) ) {
+			ircp = mtcp->cl_prq->ce_recipe;
+			break;
+		     }
+		  }
+	       }
+	       else
+		  ircp = iprq.ce_recipe;
 	    }
 	    else
 	       ircp = NIL(STRING);
@@ -444,6 +496,7 @@ ICELLPTR *nnmp;
 	       printf( "%s:  Trying prerequisite [%s] for [%s]\n", Pname,
 		       iprqh.ht_name, ic->ic_name );
 
+	    /* irpq is a temporary target cell, a stat will not be remembered. */
 	    if( !(iprq.ce_flag & F_STAT) ) Stat_target(&iprq, FALSE, FALSE);
 	 }
 
@@ -458,7 +511,9 @@ ICELLPTR *nnmp;
 	 if( meta->ce_prq )
 	    noinf |= ((meta->ce_prq->cl_prq->ce_attr)&A_NOINFER);
 	 trans = Transitive || !noinf;
-	 exists = (iprq.ce_time != (time_t)0L);
+
+	 /* If no prereq is given treat it as if it is existing. */
+	 exists = (iprq.ce_time != (time_t)0L) || (name == NIL(char));
 
 	 if( exists || (ircp != NIL(STRING)) || !name ) {
 	    match = add_iset( match, ic, meta, pdfa, idirroot, ic->ic_dmax,
@@ -484,19 +539,26 @@ ICELLPTR *nnmp;
 
 
 static char *
-buildname( tg, meta, per )
-char *tg;
+buildname( tg, meta, per )/*
+============================
+   Replace '%' with per in meta. Expand the result and return it. */ 
+char *tg; /* Current target name. */
 char *meta;
 char *per;
 {
    char    *name;
 
    name = Apply_edit( meta, "%", per, FALSE, FALSE );
+   /* Handle infered dynamic prerequisites. */
    if( strchr(name, '$') ) {
       HASHPTR m_at;
       char *tmp;
 
-      m_at = Def_macro( "@", tg, M_MULTI );
+      /* Set $@ so that a Expand() can use it and remove it afterwards. */
+      /* Is $@ already expanded? FIXME: Remove this check later. */
+      if( *DmStrPbrk( tg, "${}" ) != '\0' )
+	 Fatal("$@ [%s] not fully expanded!", tg);
+      m_at = Def_macro( "@", DO_WINPATH(tg), M_MULTI|M_EXPANDED );
       tmp = Expand( name );
 
       if( m_at->ht_value != NIL(char) ) {
@@ -504,6 +566,7 @@ char *per;
 	 m_at->ht_value = NIL(char);
       }
 
+      /* Free name if Apply_edit() did something. */
       if( name != meta ) FREE( name );
       name = tmp;
    }
@@ -718,7 +781,10 @@ ICELLPTR uset;
 
 static char *
 dump_inf_chain( ip, flag, print )/*
-====================================*/
+===================================
+Return string with infered prerequisites.
+flag == TRUE adds the top of the chain.
+print == TRUE prints to screen with number "print" and returns NULL. */
 ICELLPTR ip;
 int      flag;
 int	 print;
@@ -727,6 +793,7 @@ int	 print;
 
    if( ip == NIL(ICELL) ) return(NIL(char));
 
+   /* ip->ic_parent is the target to be build after ip. */
    tmp = dump_inf_chain(ip->ic_parent, FALSE, FALSE);
 
    if( ip->ic_meta ) {
@@ -763,6 +830,7 @@ int	 print;
 
 
 #ifdef DBUG
+static void
 _dump_dfa_stack(dfas, dfa_stack)
 DFALINKPTR dfas;
 DFASETPTR  dfa_stack;
@@ -788,6 +856,7 @@ DFASETPTR  dfa_stack;
 }
 
 
+static void
 _dump_iset( name, iset )
 char     *name;
 ICELLPTR iset;
@@ -810,7 +879,7 @@ ICELLPTR iset;
       else
 	 printf( "dfa: (nil)\n" );
 
-      printf( "sdr: %04x\n", iset->ic_setdirroot );
+      printf( "sdr: %p\n", iset->ic_setdirroot );
       _dump_dfa_stack(iset->ic_dfastack.df_set, &iset->ic_dfastack);
 
       printf( "dmax: %d\n", iset->ic_dmax );
