@@ -1,4 +1,4 @@
-/* RCS  $Id: runargv.c,v 1.4.6.3 2007/10/06 14:33:06 vq Exp $
+/* RCS  $Id: runargv.c,v 1.5.2.1 2007/11/11 23:05:48 vq Exp $
 --
 -- SYNOPSIS
 --      Run a sub process.
@@ -23,6 +23,12 @@
 --      Use cvs log to obtain detailed change logs.
 */
 
+#if defined(USE_CREATEPROCESS)
+/* MSVC6.0 and newer and MinGW use the parallel build enabled runargv(). */
+Force a compile-time blowup.
+This file should not be used, use unix/runargv.c instead.
+#endif
+
 #include <process.h>
 #include <errno.h>
 #include "extern.h"
